<?php
require_once('config.php');

class FacebookApp {  
    // + Singleton
    private static $instance = null;
    public static function instance() {
        if (!isset(self::$instance)) {           
            self::$instance = new FacebookApp();        
            self::$instance->init_session();
            self::$instance->init_api();
        }
        return self::$instance;
    }
    private function __construct() {
    }
    // - Singleton
    
    private $logged_in_to_vb = false;
    private $facebook = null;
    private $api = null;

    public function getFacebookInterface() {
        if (is_null($this->facebook)) {
            $this->init_facebook();
        }
        return $this->facebook;
    }

    public function getVbApiInterface() {
        return $this->api;
    }
    
	public function ensureFacebookUserLoggedInToVB() {       
        if (!empty($_SESSION['logged_in_to_vb']) && $_SESSION['logged_in_to_vb'] == true) {
            return;
        }
        $this->login();
	}
    public function login() {
        if(VBApiClient::instance()->getVBMajorVersion() == 4) {
            VBApiClient::instance()->facebook_login($_SESSION['facebook_session']);
        } else if(isset($_SESSION['user_credentials'])) {
            VBApiClient::instance()->login($_SESSION['user_credentials']);
        }
    }
    public function getCurrentUrl() {
        $query = '';
        if(!empty($_SERVER['QUERY_STRING'])) {
            $query = '?' . $_SERVER['QUERY_STRING'];
        }
        return basename($_SERVER['SCRIPT_NAME']) . $query;
    }

    public function requestPermissions($url = '') {   
        $dialog_url =  "http://www.facebook.com/dialog/oauth?client_id=" . FACEBOOK_APP_ID . "&redirect_uri=" . urlencode(FACEBOOK_CANVAS_URL . $url). "&scope=" . REQUEST_PERMISSIONS;
        echo("<script> top.location.href='" . $dialog_url . "'</script>");
        exit;
    }

    private function init_facebook() {
        require_once('facebook_stuff/facebook.php');
        try {
            $this->facebook = new Facebook(array(
                'appId'  => FACEBOOK_APP_ID,
                'secret' => FACEBOOK_SECRET_KEY
            ));
            
            
        } catch(Exception $e) {
            error_log('facebook error: ' . var_export($e, true));
            $this->requestPermissions();
        }
    }

    private function init_api() {
        require_once('vb_api_client.php');
        $this->api = VBApiClient::instance();
    }

    private function init_session() {
        require_once('utilities.php');
        if(!isset($_REQUEST["signed_request"])) {
            exit;
        }
        $signed_request = @$_REQUEST["signed_request"];
        list($encoded_sig, $payload) = @explode('.', $signed_request, 2); 
        $data = @json_decode(base64_decode(strtr($payload, '-_', '+/')), true);    
		if (!empty($data["user_id"])) {            
            // Before we start session, load serialized classes.
            require_once('category.php');
            session_cache_limiter('private_no_expire, must-revalidate');
            session_set_cookie_params(60 * 60 * 24 * 365 * 10);
            session_name('vbfb_' . FACEBOOK_APP_ID . '_' . $data["user_id"]);
            session_start();
            $this->init_facebook();
            $facebook = $this->getFacebookInterface();
            $signed_request = $facebook->getSignedRequest();
            
			// PHP complains if we don't do this. In the future, get the user's TZ.
			date_default_timezone_set('America/Los_Angeles');
            // Unicode string functions
            mb_internal_encoding("UTF-8");            
            
			//
			// 	If we haven't stored the facebook session, or it doesn't match the user's cookie, restart.
			//
			if(!isset($_SESSION['signed_request']) || $_SESSION['signed_request']['oauth_token'] != $signed_request['oauth_token']) {              
				try {
					$me = $facebook->api('/me');
					$my_friends = $facebook->api('/me/friends');                   
					$my_interests = $facebook->api('/me/interests');                    
					
					$my_friends_ids = get_friends_ids($my_friends);
					$_SESSION['username'] = $me['name'];                    
					$_SESSION['my_friends_ids'] = $my_friends_ids;
					$_SESSION['my_interests'] = $my_interests;
					$_SESSION['username'] = $me['name'];
					$_SESSION['profile_link'] = $me['link'];
				}                    
				catch(Exception $e) {
					error_log('facebook error: ' . var_export($e, true));
					$this->requestPermissions();
				}
                
                $_SESSION['signed_request'] = $signed_request;
				$_SESSION['facebook_session'] = $_REQUEST['signed_request'];
				$_SESSION['userid'] = $facebook->getUser();
				$_SESSION['access_token'] = $facebook->getAccessToken();
				//
				//	If we think we are logged in to VB, force a new login.
				//
                if(!empty($_SESSION['logged_in_to_vb']) && $_SESSION['logged_in_to_vb'] == true) {
                    $this->login();
                }
			}
		} else {
			error_log('signed_request: unable to find user');
			$this->requestPermissions();
		}
	}
}
?>